@@ -278,7 +278,7 @@ h2. Attendees
 
 class Boobathon(ReplApplication):
     APPNAME = 'boobathon'
-    VERSION = '1.3'
+    VERSION = '1.4'
     COPYRIGHT = 'Copyright(C) 2011-YEAR Romain Bignon'
     DESCRIPTION = 'Console application to participate to a Boobathon.'
     SHORT_DESCRIPTION = "participate in a Boobathon"
@@ -781,7 +781,7 @@ class Boobathon(ReplApplication):
                 return
 
         if not self.check_loaded_backends({'url': 'https://symlink.me'}):
-            print("Ok, so leave now, fag.")
+            print("Ok, so leave now.")
             sys.exit(0)
 
     def is_module_loadable(self, module):
