@@ -28,26 +28,68 @@ from weboob.tools.compat import unicode, long, with_metaclass, StrConv
 from weboob.tools.misc import to_unicode
 
 
-__all__ = ['UserError', 'FieldNotFound', 'NotAvailable',
+__all__ = ['UserError', 'FieldNotFound', 'NotAvailable', 'FetchError',
            'NotLoaded', 'Capability', 'Field', 'IntField', 'DecimalField',
            'FloatField', 'StringField', 'BytesField', 'BoolField',
+           'Enum', 'EnumField',
            'empty', 'BaseObject']
 
 
-def enum(**enums):
-    _values = list(enums.values())
-    _keys = list(enums.keys())
-    _items = list(enums.items())
-    _types = list((type(value) for value in enums.values()))
-    _index = {(value if not isinstance(value, dict) else next(iter(value.values()))): i
-              for i, value in enumerate(enums.values())}
+class EnumMeta(type):
+    @classmethod
+    def __prepare__(mcs, name, bases, **kwargs):
+        # in python3.6, default namespace keeps declaration order
+        # in python>=3 but <3.6, force ordered namespace
+        # doesn't work in python2
+        return OrderedDict()
 
-    enums['keys'] = _keys
-    enums['values'] = _values
-    enums['items'] = _items
-    enums['index'] = _index
-    enums['types'] = _types
-    return type('Enum', (), enums)
+    def __init__(cls, name, bases, attrs, *args, **kwargs):
+        super(EnumMeta, cls).__init__(name, bases, attrs, *args, **kwargs)
+        attrs = [(k, v) for k, v in attrs.items() if not callable(v) and not k.startswith('__')]
+        if sys.version_info.major < 3:
+            # can't have original declaration order, at least sort by value
+            attrs.sort(key=lambda kv: kv[1])
+        cls.__members__ = OrderedDict(attrs)
+
+    def __setattr__(cls, name, value):
+        super(EnumMeta, cls).__setattr__(name, value)
+        if not callable(value) and not name.startswith('__'):
+            cls.__members__[name] = value
+
+    def __call__(cls, *args, **kwargs):
+        raise ValueError("Enum type can't be instanciated")
+
+    @property
+    def _items(cls):
+        return cls.__members__.items()
+
+    @property
+    def _keys(cls):
+        return cls.__members__.keys()
+
+    @property
+    def _values(cls):
+        return cls.__members__.values()
+
+    @property
+    def _types(cls):
+        return set(map(type, cls._values))
+
+    def __iter__(cls):
+        return iter(cls.__members__.values())
+
+    def __len__(cls):
+        return len(cls.__members__)
+
+    def __contains__(cls, value):
+        return value in cls.__members__.values()
+
+    def __getitem__(cls, k):
+        return cls.__members__[k]
+
+
+class Enum(with_metaclass(EnumMeta, object)):
+    pass
 
 
 def empty(value):
@@ -56,10 +98,7 @@ def empty(value):
 
     :rtype: :class:`bool`
     """
-    for cls in (None, NotLoaded, NotAvailable):
-        if value is cls:
-            return True
-    return False
+    return value is None or isinstance(value, EmptyType)
 
 
 def find_object(mylist, error=None, **kwargs):
@@ -68,12 +107,10 @@ def find_object(mylist, error=None, **kwargs):
     kwargs.
     """
     for a in mylist:
-        found = True
         for key, value in kwargs.items():
             if getattr(a, key) != value:
-                found = False
                 break
-        if found:
+        else:
             return a
 
     if error is not None:
@@ -81,6 +118,19 @@ def find_object(mylist, error=None, **kwargs):
     return None
 
 
+def strict_find_object(mylist, error=None, **kwargs):
+    """
+    Tools to return an object with the matching parameters in kwargs.
+    Parameters with empty value are skipped
+    """
+    kwargs = {k: v for k, v in kwargs.items() if not empty(v)}
+    if kwargs:
+        return find_object(mylist, error=error, **kwargs)
+
+    if error is not None:
+        raise error()
+
+
 class UserError(Exception):
     """
     Exception containing an error message for user.
@@ -116,9 +166,9 @@ class AttributeCreationWarning(UserWarning):
     """
 
 
-class NotAvailableType(object):
+class EmptyType(object):
     """
-    NotAvailable is a constant to use on non available fields.
+    Parent class for NotAvailableType, NotLoadedType and FetchErrorType.
     """
 
     def __str__(self):
@@ -130,21 +180,28 @@ class NotAvailableType(object):
     def __deepcopy__(self, memo):
         return self
 
+    def __nonzero__(self):
+        return False
+
+    __bool__ = __nonzero__
+
+
+class NotAvailableType(EmptyType):
+    """
+    NotAvailable is a constant to use on non available fields.
+    """
+
     def __repr__(self):
         return 'NotAvailable'
 
     def __unicode__(self):
         return u'Not available'
 
-    def __nonzero__(self):
-        return False
-
-    __bool__ = __nonzero__
 
 NotAvailable = NotAvailableType()
 
 
-class NotLoadedType(object):
+class NotLoadedType(EmptyType):
     """
     NotLoaded is a constant to use on not loaded fields.
 
@@ -152,29 +209,31 @@ class NotLoadedType(object):
     it will request all fields with this value.
     """
 
-    def __str__(self):
-        return repr(self)
-
-    def __copy__(self):
-        return self
-
-    def __deepcopy__(self, memo):
-        return self
-
     def __repr__(self):
         return 'NotLoaded'
 
     def __unicode__(self):
         return u'Not loaded'
 
-    def __nonzero__(self):
-        return False
-
-    __bool__ = __nonzero__
 
 NotLoaded = NotLoadedType()
 
 
+class FetchErrorType(EmptyType):
+    """
+    FetchError is a constant to use when parsing a non-mandatory field raises an exception.
+    """
+
+    def __repr__(self):
+        return 'FetchError'
+
+    def __unicode__(self):
+        return u'Not mandatory'
+
+
+FetchError = FetchErrorType()
+
+
 class Capability(object):
     """
     This is the base class for all capabilities.
@@ -201,6 +260,7 @@ class Field(object):
         self.types = ()
         self.value = kwargs.get('default', NotLoaded)
         self.doc = doc
+        self.mandatory = kwargs.get('mandatory', True)
 
         for arg in args:
             if isinstance(arg, type) or isinstance(arg, str):
@@ -282,7 +342,7 @@ class StringField(Field):
 
 class BytesField(Field):
     """
-    A field which accepts only :class:`str` strings.
+    A field which accepts only :class:`bytes` strings.
     """
 
     def __init__(self, doc, **kwargs):
@@ -294,6 +354,19 @@ class BytesField(Field):
         return bytes(value)
 
 
+class EnumField(Field):
+    def __init__(self, doc, enum, **kwargs):
+        if not issubclass(enum, Enum):
+            raise TypeError('invalid enum type: %r' % enum)
+        super(EnumField, self).__init__(doc, *enum._types, **kwargs)
+        self.enum = enum
+
+    def convert(self, value):
+        if value not in self.enum._values:
+            raise ValueError('value %r does not belong to enum %s' % (value, self.enum))
+        return value
+
+
 class _BaseObjectMeta(type):
     def __new__(cls, name, bases, attrs):
         fields = [(field_name, attrs.pop(field_name)) for field_name, obj in list(attrs.items()) if isinstance(obj, Field)]
@@ -362,7 +435,7 @@ class BaseObject(with_metaclass(_BaseObjectMeta, StrConv, object)):
         """
         Return True if the object is completed.
 
-        It is usefull when the object is a field of an other object which is
+        It is useful when the object is a field of an other object which is
         going to be filled.
 
         The default behavior is to iter on fields (with iter_fields) and if
@@ -507,7 +580,7 @@ class BaseObject(with_metaclass(_BaseObjectMeta, StrConv, object)):
         return self
 
     def __setstate__(self, state):
-        self._fields = deepcopy(self._fields) # because yaml does not call __init__
+        self._fields = deepcopy(self._fields)  # because yaml does not call __init__
         for k in state:
             setattr(self, k, state[k])
 
@@ -518,7 +591,7 @@ class BaseObject(with_metaclass(_BaseObjectMeta, StrConv, object)):
 
 class Currency(object):
     CURRENCIES = OrderedDict([
-        (u'EUR', (u'€', u'EURO')),
+        (u'EUR', (u'€', u'EURO', u'EUROS')),
         (u'CHF', (u'CHF',)),
         (u'USD', (u'$',)),
         (u'GBP', (u'£',)),
@@ -540,6 +613,7 @@ class Currency(object):
         (u'MYR', (u'RM',)),
         (u'HUF', (u'Ft',)),
         (u'HKD', (u'HK$',)),
+        (u'TWD', (u'NT$',)),
         (u'QAR', (u'QR',)),
         (u'MAD', (u'MAD',)),
         (u'ARS', (u'ARS',)),
@@ -549,14 +623,23 @@ class Currency(object):
         (u'BHD', (u'BHD',)),
         (u'SEK', (u'SEK',)),
         (u'DKK', (u'DKK',)),
-        (u'LUF', (u'F', u'fr.', u'LUF',)),
+        (u'LUF', (u'LUF',)),
         (u'KZT', (u'KZT',)),
         (u'PLN', (u'PLN',)),
         (u'ILS', (u'ILS',)),
         (u'THB', (u'THB',)),
+        (u'INR', (u'₹', u'INR')),
+        (u'PEN', (u'S/',)),
+        (u'IDR', (u'Rp',)),
+        (u'KWD', (u'KD',)),
+        (u'KRW', (u'₩',)),
+        (u'CZK', (u'Kč',)),
+        (u'EGP', (u'E£',)),
+        (u'ISK', (u'Íkr', u'kr')),
+        (u'XPF', (u'XPF',)),
     ])
 
-    EXTRACTOR = re.compile(r'[\d\s,\.\-]', re.UNICODE)
+    EXTRACTOR = re.compile(r'[()\d\s,\.\-]', re.UNICODE)
 
     @classmethod
     def get_currency(klass, text):
@@ -577,12 +660,13 @@ class Currency(object):
         None
         """
         curtexts = klass.EXTRACTOR.sub(' ', text.upper()).split()
-        for curtext in curtexts:
-            for currency, symbols in klass.CURRENCIES.items():
-                if curtext in currency:
+
+        for currency, symbols in klass.CURRENCIES.items():
+            for curtext in curtexts:
+                if curtext == currency:
                     return currency
                 for symbol in symbols:
-                    if curtext in symbol or symbol in curtext:
+                    if curtext == symbol:
                         return currency
         return None
 
