@@ -36,6 +36,8 @@ from weboob.tools.config.iconfig import ConfigError
 from weboob.exceptions import FormFieldConversionWarning
 from weboob.tools.log import createColoredFormatter, getLogger, DEBUG_FILTERS, settings as log_settings
 from weboob.tools.misc import to_unicode, guess_encoding
+from weboob.tools.compat import unicode
+
 from .results import ResultsConditionError
 
 __all__ = ['Application']
@@ -307,7 +309,7 @@ class Application(object):
         else:
             res = getattr(backend, function)(*args, **kwargs)
 
-        if hasattr(res, '__iter__'):
+        if hasattr(res, '__iter__') and not isinstance(res, (bytes, unicode)):
             return self._do_complete_iter(backend, count, selected_fields, res)
         else:
             return self._do_complete_obj(backend, selected_fields, res)
@@ -316,7 +318,7 @@ class Application(object):
         """
         Handler for an exception inside the CallErrors exception.
 
-        This method can be overrided to support more exceptions types.
+        This method can be overridden to support more exceptions types.
         """
 
         # Ignore this error.
@@ -342,6 +344,8 @@ class Application(object):
         :param ignore: Exceptions to ignore
         :type ignore: tuple[:class:`Exception`]
         """
+        err = 0
+
         ask_debug_mode = False
         for backend, error, backtrace in errors.errors:
             if isinstance(error, ignore):
@@ -349,18 +353,28 @@ class Application(object):
             elif self.bcall_error_handler(backend, error, backtrace):
                 ask_debug_mode = True
 
+            if not isinstance(error, MoreResultsAvailable):
+                err = 1
+
         if ask_debug_mode:
             print(debugmsg, file=self.stderr)
 
+        return err
+
+    def _shell_completion_items(self):
+        items = set()
+        for ol in [self._parser.option_list] + [og.option_list for og in self._parser.option_groups]:
+            for option in ol:
+                if option.help is not optparse.SUPPRESS_HELP:
+                    items.update(str(option).split('/'))
+        items.update(self._get_completions())
+        return items
+
     def parse_args(self, args):
         self.options, args = self._parser.parse_args(args)
 
         if self.options.shell_completion:
-            items = set()
-            for option in self._parser.option_list:
-                if option.help is not optparse.SUPPRESS_HELP:
-                    items.update(str(option).split('/'))
-            items.update(self._get_completions())
+            items = self._shell_completion_items()
             print(' '.join(items))
             sys.exit(0)
 
@@ -427,10 +441,12 @@ class Application(object):
             logging.root.addHandler(handler)
 
     def setup_nss(self):
-        from weboob.browser.nss import init_nss, inject_in_urllib3, create_cert_db
+        from weboob.browser.nss import (
+            init_nss, inject_in_urllib3, create_cert_db, certificate_db_filename,
+        )
 
         path = self.CONFDIR
-        if not os.path.exists(os.path.join(path, 'cert8.db')):
+        if not os.path.exists(os.path.join(path, certificate_db_filename())):
             create_cert_db(path)
         init_nss(path)
         inject_in_urllib3()
@@ -497,9 +513,11 @@ class Application(object):
                 sys.exit(1)
             except CallErrors as e:
                 try:
-                    app.bcall_errors_handler(e)
+                    ret = app.bcall_errors_handler(e)
                 except KeyboardInterrupt:
                     pass
+                else:
+                    sys.exit(ret)
                 sys.exit(1)
             except ResultsConditionError as e:
                 print('%s' % e, file=cls.stderr)
